#ifndef HARDWARE_H
#define HARDWARE_H
/*
Compatible with the F11xx FET KIT from TI.
The JTAG interface is used as level converter for a SPI master controlled
by the PC. Therefore the SPI pins were choosen that way.

                MSP430F1121
             -----------------
         /|\|              XIN|-
          | |                 |
          --|RST          XOUT|-
            |                 |
      NC <--|P2.0         P1.0|--> LED
      NC <--|P2.1         P1.1|--> FREQ_OUT
      NC <--|P2.2         P1.2|--> PWM_OUT1
      NC <--|P2.3         P1.3|--> PWM_OUT2
      NC <--|P2.4         P1.4|--> TCK      CLK   \
      NC <--|P2.5         P1.5|--> TMS      /CS   | SPI lines
            |             P1.6|--> TDI      SIMO  |
            |             P1.7|--> TDO      SOMI  /
            |                 |
             -----------------

The timer is set up as PWM. The base frequency (FREQ_OUT) and the two PWM
signals (PWM_OUTx) are output on the pins P1.1 to P1.3. The FET kit also
has a LED on P1.0 which is high active.

(C) 2004 Chris Liechti <cliechti@gmx.net>
this is distributed under a free software license, see license.txt

http://mspgcc.sf.net
*/

#include <io.h>
#include <signal.h>

//PINS
//--- PORT1 ---
#define LED             BIT0
#define FREQ_OUT        BIT1
#define PWM_OUT1        BIT2
#define PWM_OUT2        BIT3
#define TCK             BIT4
#define TMS             BIT5
#define TDI             BIT6
#define TDO             BIT7

#define P1OUT_INIT      TDO
#define P1SEL_INIT      FREQ_OUT|PWM_OUT1|PWM_OUT2
#define P1DIR_INIT      FREQ_OUT|PWM_OUT1|PWM_OUT2|TDO|LED

#define P1IE_INIT       0
#define P1IES_INIT      0

#define LED_OFF()       (P1OUT &= ~LED)
#define LED_ON()        (P1OUT |=  LED)

//--- PORT2 ---
#define NC20            BIT0
#define NC21            BIT1
#define NC22            BIT2
#define NC23            BIT3
#define NC24            BIT4
#define NC25            BIT5

#define P2OUT_INIT      0
#define P2SEL_INIT      0
#define P2DIR_INIT      NC20|NC21|NC22|NC23|NC24|NC25

#define CAPD_INIT       0

#define P2IE_INIT       0
#define P2IES_INIT      0

////
#define IE1_INIT        0
#define IE2_INIT        0
#define ME1_INIT        0
#define ME2_INIT        0


#define WDTCTL_INIT     WDTPW|WDTHOLD

#define BCSCTL1_INIT    XT2OFF|RSEL2|RSEL1|RSEL0
#define BCSCTL2_INIT    0
#define DCOCTL_INIT     0xff

//~ #define TACTL_INIT      TASSEL_ACLK|TACLR
#define TACTL_INIT      TASSEL_SMCLK|TACLR

#endif //HARDWARE_H
