;******************************************************************************
;
; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted 
; by Texas Instruments is distributed as "freeware".  You may, 
; only under TI's copyright in the Program, use and modify the 
; Program without any charge or restriction.  You may 
; distribute to third parties, provided that you transfer a 
; copy of this license to the third party and the third party 
; agrees to these terms by its first use of the Program. You 
; must reproduce the copyright notice and any other legend of 
; ownership on each copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains 
; copyrighted material, trade secrets and other TI proprietary 
; information and is protected by copyright laws, 
; international copyright treaties, and trade secret laws, as 
; well as other intellectual property laws.  To protect TI's 
; rights in the Program, you agree not to decompile, reverse 
; engineer, disassemble or otherwise translate any object code 
; versions of the Program to a human-readable form.  You agree 
; that in no event will you alter, remove or destroy any 
; copyright notice included in the Program.  TI reserves all 
; rights not specifically granted under this license. Except 
; as specifically provided herein, nothing in this agreement 
; shall be construed as conferring by implication, estoppel, 
; or otherwise, upon you, any license or other right under any 
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.
;
;******************************************************************************
; MSP430F149 Voice Recording in FLASH Demonstration
; Author    Murugavel Raju
;           Texas Instruments, Inc
; Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
#include <signal.h>
#include <io.h>
;******************************************************************************
;          CPU runs from XT2 @3.58MHz
;	 
;          Constants Definition
           .equ   FS,0x01
           .equ   Memstart,0x1401
           .equ   Memend,0xFE01
;******************************************************************************
            .section    .text
RESET:     mov.w   #__stack,R1              ; define stackpointer
           call    #Init_Sys                ; Initialize System
           mov.b   &P1IN,R5                 ; Test P1.1 for mode
           and.b   #BIT4,R5                 ; Mask Bit4 to test 'Record' button
           jnz     Play                     ; Jump to Play if not pressed
           call    #Erase                   ; Flash erase subroutine           
           xor.w   #FXKEY+WRT,&FCTL1        ; Enable FLASH write for recording
           bis.b   #BIT0,&P1OUT             ; Led ON     
Play:      eint                             ; Enable interrupts
           mov.w   #Memstart,R14            ; Start memory address to R14 
Mainloop:  jmp     Mainloop                 ; Keep looping, only ISR's 
                                            ; are serviced 
;******************************************************************************
TB7_ISR:    ;Timer B7 ISR samples and stores during RECORD 
            ;and send out data during PLAY
;******************************************************************************
           bit.b   #BIT4,R5                 ; Test for mode button
           jnz     Play1                    ; Jump to Play1 if not pressed           
           bic.w   #ADC12SC,&ADC12CTL0      ; Start conversion
Conv_tst:  bit.w   #ADC12BUSY,&ADC12CTL1    ; Test for conversion complete
           jc      Conv_tst                 ; Loop till conversion complete
           bis.w   #ADC12SC,&ADC12CTL0      ; back to sample mode
           mov.w   &ADC12MEM0,0(R14)        ; Write word to FLASH           
           incd.w  R14                      ; Increment pointer
           cmp.w   #Memend,R14              ; Check if memarray full
           jnz     Proceed                  ; Jump to Proceed if not full
           bic.b   #BIT0,&P1OUT             ; Led OFF if record memory array full
           xor.w   #FXKEY+WRT,&FCTL1        ; Disable FLASH write 
           xor.w   #FXKEY+LOCK,&FCTL3       ; Lock FLASH memory     
           jmp     RESET                    ; Loop again (will Playback if 
                                            ; button released) 
Play1:     mov.w   @R14,R15                 ; Read data from memory pointer 
           incd.w  R14                      ; Double increment pointer to point
                                            ; to next data word 
           cmp.w   #Memend,R14              ; Check if end of memarray 
           jnz     Go_on                    ; Jump to Go_on if not end
           jmp     RESET                    ; Loop again if end
Go_on:     
L1:        bit.b   #UTXIFG0,&IE1            ; Loop to wait until previous
           jc      L1                       ; transmission done and buffer empty
           bis.b   #FS,&P3OUT               ; Pulse Frame sync
           bic.b   #FS,&P3OUT               ; to start loading TLV5516 with 
                                            ; next data word	
           swpb    R15                      ; Swap bytes in R15
           mov.b   R15,&TXBUF0              ; High byte to SPI TXBUF
           swpb    R15                      ; Swap bytes in R15
           mov.b   R15,&TXBUF0              ; Low byte to SPI TXBUF 	 
Proceed:   reti                             ; Return from ISR
;******************************************************************************
Init_Sys:   ;Setup Peripherals
;******************************************************************************
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupBC:    bic.b   #XT2OFF,&BCSCTL1        ; XT2 ON
            call    #Delay                  ; Delay for crystal stabilization            
            mov.b   #SELM1+SELS,&BCSCTL2    ; MCLK=SMCLK=XT2CLK
            bic.b   #OFIFG,&IFG1            ; Clear OFIFG            
SetupP1:    mov.b   #0,&P1OUT               ; Clear P1 output register
            bis.b   #0xFF,&P1DIR            ; P1.0 for LED output  
                                            ; and unused pins as o/p's
            bic.b   #BIT4,&P1DIR            ; For switch input            
SetupP2:    mov.b   #0,&P2OUT               ; Clear P2 output register
            bis.b   #0xFF,&P2DIR            ; Unused pins as o/p's                                    
SetupP3:    bis.b   #0x0A,&P3SEL            ; P3.1 & P3.3 SPI option select
            bis.b   #FS,&P3OUT              ; FS set 
            bis.b   #0xFE+FS,&P3DIR         ; P3.0,3.1,3.3 & unsued pins o/p dir.            
SetupP4:    mov.b   #0,&P4OUT               ; Clear P4 output register
            bis.b   #0xFF,&P4DIR            ; Unused pins as o/p's            
SetupP5:    mov.b   #0h,&P5OUT              ; Clear P5 output register
            bis.b   #0xFF,&P5DIR            ; Unused pins as o/p's                                     
SetupP6:    bis.b   #BIT0,&P6SEL            ; P6.0 = ADC12 A0 input
            mov.b   #0,&P6OUT               ; P6 output pins to reset
            bis.b   #0xFE,&P6DIR            ; P6.1 to 6.7 outputs (unused)            
SetupADC:   Call    #ADCset                 ; Initialize ADC12
SetupUSART: bis.b   #0x40,&ME1              ; Enable USART module
            mov.b   #CHAR+SYNC+MM,&U0CTL    ; 8-bit SPI Master
            mov.b   #CKPL+SSEL1+SSEL0+STC,&U0TCTL 
                                            ; SMCLK for TX, 3-pin mode
            mov.b   #0x02,&U0BR0            ; SMCLK/2 for baud rate
            clr.b   &U0BR1                  ; 
            clr.b   &U0MCTL                 ; Clear Modulation            		 
SetupCCR0:  bis.w   #CCIE,&TBCCTL0
            mov.w   #649,&TBCCR0            ; Initialize TBCCR0 for sampling 
                                            ; frequency of 5.5Khz            
SetupTB7:   bis.w   #TBSSEL1+MC0,&TBCTL     ; TimerB in UP mode
SetupFlash: xor.w   #FXKEY+FN2+FN1+FN0,&FCTL2
                                            ; Set FLASH timing generator 447.5Khz            
            ret                             ; Return from subroutine
;******************************************************************************                                         	 
ADCset:     ;Initialize ADC12, Vcc as ADC Reference Voltage
            ;Single-channel (A0) single-conversion mode  
;****************************************************************************** 
            bis.w   #ADC12ON+ADC12SC+ENC,&ADC12CTL0 
                                            ; Turn ON ADC12 & S/H in sample
            call    #Delay                  ; Delay for stabilization
            bis.w   #ADC12SSEL_1+ADC12SSEL_2,&ADC12CTL1
                                            ; ADC12 Clock=SMCLK
            ret                             ; Return from subroutine 
;******************************************************************************                                         	 
Erase:      ;Initialize FCTL & Erase FLASH for new recording 
;******************************************************************************              
            dint                            ; Disable interrupts
            xor.w   #FXKEY+LOCK,&FCTL3      ; Unlock FLASH for write                                                         
Test_Busy1: bit.w   #BUSY,&FCTL3            ; Check BUSY flag
            jnz     Test_Busy1              ; Loop till not busy
            mov.w   #0x1400,R13             ; Start of record memory array to R13
NextSeg:    mov.w   #(FWKEY+ERASE),&FCTL1   ; Set 
            clr.b   0(R13)                  ; Perform a dummy write to activate
                                            ; segment erase 
Test_Busy2: bit.w   #BUSY,&FCTL3            ; Check BUSY flag
            jnz     Test_Busy2              ; Loop till not busy 
            add.w   #0x200,R13              ; Point to next segments
            cmp.w   #0xFE00,R13             ; Check if all segments are erased
            jnz     NextSeg                 ; If not proceed erasing next segment                                               
            ret                             ; Return from subroutine            
;****************************************************************************** 
Delay:      ;Software delay
;****************************************************************************** 
            push.w  #0xFFFF                 ; Delay to TOS
DL1:        dec.w   0(R1)                   ; Decrement TOS
            jnz     DL1                     ; Delay over?
            incd.w  R1                      ; Clean TOS
            ret                             ; Return from subroutine      
;****************************************************************************** 
; MSP430F14x Interrupt vectors             
;****************************************************************************** 
            .section    .vectors
            .org    TIMERB0_VECTOR,0xFF
WDT_VEC:    .word   TB7_ISR                 ; Timer B7 ISR
            .org    RESET_VECTOR,0xFF
RESET_VEC:  .word   RESET                   ; POR, ext. Reset, Watchdog
