; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
; INTELLECTUAL PROPERTY RIGHTS WIT REGARD TO THE PROGRAM OR 
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
; OF THE POSSIBILITY OF SUC DAMAGES, ARISING IN ANY WAY OUT 
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted 
; by Texas Instruments is distributed as "freeware".  You may, 
; only under TI's copyright in the Program, use and modify the 
; Program without any charge or restriction.  You may 
; distribute to third parties, provided that you transfer a 
; copy of this license to the third party and the third party 
; agrees to these terms by its first use of the Program. You 
; must reproduce the copyright notice and any other legend of 
; ownership on eac copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains 
; copyrighted material, trade secrets and other TI proprietary 
; information and is protected by copyright laws, 
; international copyright treaties, and trade secret laws, as 
; well as other intellectual property laws.  To protect TI's 
; rights in the Program, you agree not to decompile, reverse 
; engineer, disassemble or otherwise translate any object code 
; versions of the Program to a human-readable form.  You agree 
; that in no event will you alter, remove or destroy any 
; copyright notice included in the Program.  TI reserves all 
; rights not specifically granted under this license. Except 
; as specifically provided herein, nothing in this agreement 
; shall be construed as conferring by implication, estoppel, 
; or otherwise, upon you, any license or other right under any 
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.

#include <signal.h>
#include <io.h>
;******************************************************************************
;
;           Implement a direct thermocouple interface wit the ADS1240 and
;           MSP430.
;
;           Mike Mitchell
;           MSP430 Applications
;           May 10, 2001
;           Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;           
;******************************************************************************

; Other Definitions

#define   ADC_HI  R14
#define   ADC_LO  R10
#define   TXBUF   R7 
#define   RXBUF   R6 
#define   BITCNT  R5

; LCD segment Definitions
          .EQU     a,0x01
          .EQU     b,0x02
          .EQU     c,0x10
          .EQU     d,0x04
          .EQU     e,0x80
          .EQU     f,0x20
          .EQU     g,0x08
          .EQU     h,0x40

; ADS1240 serial communication definitions
          .EQU     Dout,0x04
          .EQU     Din,0x08
          .EQU     SCLK,0x10
          .EQU     CS,0x40

; ADS1240 Register Definitions (Not complete list, see ADS1240 datasheet)
          .EQU     SETUP,0x00
          .EQU     MUX,0x01
          .EQU     ACR,0x02

; ADS1240 Commands (Not complete list, see ADS1240 datasheet)
          .EQU     RDATA,0x01
          .EQU     WREG,50      ; The opcode for WREG is <0101 reg xxxx number>
          .EQU     RREG,10      ; The opcode for RREG is <0001 reg xxxx number>
          .EQU     SELFCAL,0xF0
          .EQU     SELFOCAL,0xF1
          .EQU     SELFGCAL,0xF2
          .EQU     SYSOCAL,0xF3
          .EQU     SYSGCAL,0xF4
          .EQU     RST,0xFE


;****************************************************************************** 
            .section    .data
TC_ADC_HI:        .DS      2     ; Thermocouple ADC reading, hig word
TC_ADC_LO:        .DS      1     ; Thermocouple ADC reading, low byte
TR_ADC_LO:        .DS      1     ; Thermistor ADC reading, low byte
TR_ADC_HI:        .DS      2     ; Thermistor ADC reading, hig word
TC_TMP:           .DS      2     ; Thermocouple temperature
TR_TMP:           .DS      2     ; Thermistor temperature
ABS_TMP:          .DS      2     ; Absolute temperature (TR_TMP+TC_TMP)
TC_NEG:           .DS      1     ; Used for keeping track of negative TC value

;****************************************************************************** 
            .section    .text
;  Setup uC and ADC
RESET:      mov     #__stack,R1         ; define stackpointer
SetupWDT:   mov     #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupFLL:   mov.b   #0x2D,&SCFQCTL      ; Set N=45. 46x32KHz = 1.5Mhz
                                        ; Set clock for ADS1240 to be above
                                        ; min of 1MHz for some margin.
SetupLCD:   mov.b   #LCDON+LCD4MUX+LCDP2,&LCDCTL   ; LCD 4Mux, S0-S17
SetupBT:    mov.b   #BTDIV+BTFRFQ1,&BTCTL  ; LCD freq.
ClearLCD:   mov     #15,R15             ; Clear 15 bytes of LCD RAM 
Clear1:     clr.b   LCDMEM-1(R15)       ; Clear LCD RAM  
            dec     R15                 ; All 15 bytes of LCD RAM?
            jnz     Clear1              ; Not done?
            
SetupIO:    mov.b   #0xFF,&P5SEL        ; Select LCD functions
            mov.b   #BIT0+BIT1,&P5DIR   ; Set output direction
            mov.b   #BIT1,&P1SEL        ; Select MCLK output for pin
            mov.b   #BIT0,&P1IES        ; Select H/L transition for interrupt
                                        ; DRDY\ is connected pin P1.0.
            mov.b   #0,&P1IFG           ; The previous instruction could have
                                        ; set the P1.0 IFG, so clear it.
            mov.b   #BIT0,&P1IE         ; Enable interrupt on P1.0
            mov.b   #BIT1+Din+SCLK+CS,&P1DIR
                                        ; Set direction for serial comm. pins
                                        ; MCLK used as OSC input to ADS1240
            mov.b   #CS,&P1OUT          ; Set CS High, everything else low

SetupIO2:   ; Set all unused IO ports to output direction
            bis.b   #BIT5+BIT6+BIT7,&P1DIR
            bis.b   #0xFF,&P2DIR
            bis.b   #BIT0+BIT1+BIT2,&P3DIR
            bis.b   #0xFF,&P6DIR                        

Delay1:     mov    #0xFFF,R15           ; Delay to allow FLL+ to 
loop1:      dec    R15                  ; sync up MCLK
            jnz    loop1

            Call   #SetupADC            ; Call setup routine for ADC

Mainloop:   bis.w   #LPM0,R2            ; Enter LPM0
            call    #ReadTC             ; Read Thermocouple
            bis.w   #LPM0,R2            ; Enter LPM0
            call    #ReadTR             ; Read Thermistor      
            call    #Get_TR_temp        ; Determine temperature wit tables
                                        ; and interpolation
            call    #Get_TC_temp        ; Determine temperature wit tables
                                        ; and interpolation
            call    #Get_ABS_temp       ; Determine absolute temperature
            call    #DISPLCD            ; Display
            jmp     Mainloop            ; Return to low power mode

;******************************************************************************
SetupADC:   ; Routine to setup the ADS1240 ADC
;******************************************************************************
            bic.b   #CS,&P1OUT          ; Take CS\ low to select the ADC.

; Issue reset command
            mov.b   #RST,TXBUF          ; Move Reset command to TX buffer (R15)
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out the Reset command
            call    #SCLKdlay           ; Delay for SCLK

; Set PGA to 16X to amplify thermocouple voltage
            mov.b   #WREG+SETUP,TXBUF   ; Move write register command plus reg
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            mov.b   #2,TXBUF            ; Second part of write reg command
                                        ; Denotes going to write 3 registers total:
                                        ; Setup, Mux, and ACR
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            mov.b   #4,TXBUF            ; Set PGA to 16X
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; Select channels 0,1,common for +,- inputs.  This is for the thermocouple
            mov.b   #1,TXBUF            ; Select analog channels for conversion
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; Setup up Data rate wit ACR register
            mov.b   #2,TXBUF            ; Select slowest data rate for DRDY\.
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; enable interrupts
            eint                        ; Enable interrupts
                                        ; Interrupts needed at this point to
                                        ; delay for SELFCAL command.

; Issue SELFCAL command
            mov.b   #SELFCAL,TXBUF      ; Move Self cal command to TX buffer
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out the command
            bis     #LPM0,R2            ; Go to sleep until DRDY\ goes low.
                                        ; At that point the selfcal is done.
            bis.b   #CS,&P1OUT          ; Take CS\ back hig again
            ret                         ; Done setting up ADC.


;******************************************************************************
ReadTC:     ; Routine to read thermocouple value
;******************************************************************************
            bic.b   #CS,&P1OUT          ; Take CS\ low to select ADC.
           
; Transfer read data command
            mov     #RDATA,TXBUF        ; Move read command to TX buffer
            mov.b   #8,BITCNT           ; Set Bit count value
            call    #TXLoop             ; Shift out the read command
            call    #SCLKdlay           ; Delay for SCLK

; Get the data from the ADC
            mov     #0x10,BITCNT        ; Set bit counter to 16 bits to shift
                                        ; in the upper 16 bits of ADC value.
            call    #RXLoop             ; Call shift routine
            mov     RXBUF,TC_ADC_HI     ; Save off hig word of data.
            mov     #0x08,BITCNT        ; Set bit counter to 8 to shift in
                                        ; lower 8 bits of ADC data.
            call    #RXLoop             ; Call shift routine
            mov.b   RXBUF,TC_ADC_LO     ; Save off low byte of ADC data

; Now set mux and gain for thermistor for next ADC read.
            mov.b   #WREG+SETUP,TXBUF   ; Move write register command plus reg
            mov.b   #0x08,BITCNT        ; Set bit count value
            call    #TXLoop             ; Shift out command
            mov.b   #0x01,TXBUF         ; Second part of write reg command
            mov.b   #0x08,BITCNT        ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; delay for SCLK

; Set PGA to 1X.  The Thermistor needs no amp.
            mov.b   #0x00,TXBUF         ; Set PGA to 1X
            mov.b   #0x08,BITCNT        ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; Select channels 2,common for +,- inputs.
            mov.b   #0x28,TXBUF         ; Select analog channels for conversion
            mov.b   #0x08,BITCNT        ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; Done reading values, set CS\ hig and return
            bis.b   #CS,&P1OUT           ; Take CS\ back high.
            ret

;******************************************************************************
ReadTR:     ; Routine to read thermistor value
;******************************************************************************
            bic.b   #CS,&P1OUT          ; Take CS\ low to select ADC.

; First read the data for the thermistor
; Transfer read data command
            mov     #RDATA,TXBUF        ; Move read command to TX buffer
            mov.b   #8,BITCNT           ; Set Bit count value
            call    #TXLoop             ; Shift out the read command
            call    #SCLKdlay           ; Delay for SCLK;

; Get the data from the ADC
            mov     #0x10,BITCNT        ; Set bit counter to 16 bits to shift
                                        ; in the upper 16 bits of ADC value.
            call    #RXLoop             ; Call shift routine
            mov     RXBUF,TR_ADC_HI     ; Save off hig word of data.
            mov     #8,BITCNT           ; Set bit counter to 8 to shift in
                                        ; lower 8 bits of ADC data.
            call    #RXLoop             ; Call shift routine
            mov.b   RXBUF,TR_ADC_LO     ; Save off low byte of ADC data

; Now set mux and gain for thermocouple for next ADC read.
            mov.b   #WREG+SETUP,TXBUF   ; Move write register command plus reg
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            mov.b   #1,TXBUF            ; Second part of write reg command.
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command

; Set PGA to 16X to amplify thermocouple voltage.
            mov.b   #0x04,TXBUF         ; Set PGA to 16X
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; Select channels 0,1,common for +,- inputs.
            mov.b   #1,TXBUF            ; Select analog channels for conversion
            mov.b   #8,BITCNT           ; Set bit count value
            call    #TXLoop             ; Shift out command
            call    #SCLKdlay           ; Delay for SCLK

; Done reading values, set CS\ hig and return
            bis.b   #CS,&P1OUT           ; Take CS\ back high.
            ret

;******************************************************************************
TXLoop:     ; Routine to transmit to the ADS1240 ADC
;******************************************************************************
            bis.b   #SCLK,&P1OUT        ; Set SCLK high
            rla.b   TXBUF               ; Rotate TXBUF throug C
            jc      Set_               ; Jump if C is High
Set_L:      bic.b   #Din,&P1OUT         ; Set Dout Low
            jmp     A
Set_:       bis.b   #Din,&P1OUT         ; Set Dout High
A:          bic.b   #SCLK,&P1OUT        ; Set SCLK Low to latc data into ADC
            dec.b   BITCNT              ; Decrement Bit counter
            jnz     TXLoop              ; Continue if not done

            ret

;******************************************************************************
RXLoop:     ; Routine to shift in ADC data
;******************************************************************************
            mov     #0,RXBUF            ; Clear buffer    
L1:         bis.b   #SCLK,&P1OUT        ; Set SCLK high.
            nop                         ; Short delay
            bic.b   #SCLK,&P1OUT        ; Set SCLK low
            bit.b   #Dout,&P1IN         ; Latc data into Carry bit
            rlc     RXBUF               ; C -> Receive buffer
            dec     BITCNT              ; Decrement bit counter
            jnz     L1                  ; Continue if not done
            ret

;******************************************************************************
SCLKdlay:   ; Delay loop to meet spec. for SCLK inactive
;******************************************************************************
            mov     #0xF,R7             ; load delay value
L2:         dec     R7                  ; decrement
            jnz     L2                  ; repeat until zero
            ret

;******************************************************************************
Get_TR_temp:      ; Determine temperature of thermistor
;                 ; using tables and interpolation
;                 ; 
;****************************************************************************** 
            dint                            ; Disable interrupts.
            mov     TR_ADC_HI,R14           ; Move hig word of TR data to R14
                                            ; Not using low byte in this app.
                                            ; Only using upper 16-bits.
            mov     #0,R5                   ; Use R5 as table pointer
            mov     #0,R13                  ; Use R13 for temperature value
            cmp     TR_Temps(R5),R14        ; Compare A/D hig word to table
            jge     End_TR_Temp             ; If Thermistor value is greater than
                                            ; first value in table, then it's temp
                                            ; is too low, so end.  If equal, then
                                            ; then temp is = 0.
            incd    R5                      ; Point to second value in table
CMPloop1:   cmp     TR_Temps(R5),R14        ; Compare again
            jeq     End_LU_1                ; If equal, need to add one to temp 
                                            ; value then end.
            jge     End_TR_lookup           ; If greater, then end routine
            incd    R5                      ; Point to next value in table
            inc     R13                     ; Add one to temp value
            jmp     CMPloop1                ; Repeat loop

End_LU_1:   inc     R13                     ; Add one to temp value
            incd    R5                      ; Also need to increment pointer

End_TR_lookup:
; Resolved to nearest degree, now multiply by 10 and interpolate tenths.
; After tenths are determined, simply add it.  Then will have temp to nearest
; tent in binary format.
            rla     R13                     ; rotate left
            mov     R13,R12                 ; copy
            rla     R12                     ; rotate the copy two more times
            rla     R12                     ;
            add     R12,R13                 ; then add.
            mov     R13,TR_TMP              ; Save off to RAM
            
; Interpolate to tenths. Tenths = ((higher-ADCvalue)x10)/(higher-lower)

            mov     TR_Temps(R5),R13        ; Move table value to R13
            decd    R5                      ; Point to previous value in table
            mov     TR_Temps(R5),R15        ; Now, the higher is in R15
                                            ; the ADC value is in R14
                                            ; and the lower is in R13
            sub     R13,R15                 ; Get delta between lower and higher
            mov     TR_Temps(R5),R13        ; Move higher table value to R13
            sub     R14,R13                 ; Get delta between ADC value and higher
                                            ;             
; Multiply delta between ADC value and higher table value by 10
            rla     R13                     ; rotate left
            mov     R13,R12                 ; copy
            rla     R12                     ; rotate the copy two more times
            rla     R12                     ; then add.
            add     R12,R13                 ; Done wit x10.  Now need to divide

; Divide (higher-ADCvalue)x10 by delta between lower and upper to get tenths
;    Dividend = (higher-ADCvalue)x10, whic is in R13
;    Divisor  = (higher-lower), whic is in R15
            mov     #0,R12                  ; Use R12 as counter for subtractions
            cmp     R15,R13                 ; Compare to see if R13>R15.
            jl      remainder               ; If not, no division, just remainder
Divide1:    sub     R15,R13                 ; Dividend-Divisor -> dividend
            inc.b   R12
            cmp     R15,R13                 ; Is Dividend>Divisor?
            jge     Divide1                 ; Yes, subtract and increment again
remainder:  rla     R13                     ; Multiply remainder by two
            cmp     R15,R13                 ; Is Dividend>Divisor?
            jl      EndDiv1                 ; If not, return
            inc.b   R12                     ; If so, incrment counter to round up

EndDiv1:    ;    Division now done, so Add tenths (R12) to the whole number (TR_TMP)
            add    R12,TR_TMP               ; Now have xx.x (in binary) for the
                                            ; thermistor temperature.
            ret                             ; Done wit TR temp. Note interrupts not
                                            ; re-enabled until end of Get_ABS_temp
            
End_TR_Temp:
            mov     #0,TR_TMP               ; If get here, TR temp is 0 or too low
                                            ; So store 00.0 as the temp.
            ret                             ; Return.  Note interrupts not re-
                                            ; enabled until end of Get_ABS_temp

;******************************************************************************
Get_TC_temp:      ; Determine temperature of thermocouple
;                 ; using tables and interpolation
;                 ; 
;****************************************************************************** 
            mov     TC_ADC_HI,R14           ; Move hig word of TC data to R14
                                            ; Not using low byte in this app.
                                            ; Only using upper 16-bits.
            mov     #0,R5                   ; Use R5 as table pointer
            mov     #0xFFD7,R13             ; Use R13 for temperature value
                                            ; Preload it wit -41 deg C.
            cmp     TC_Temps(R5),R14        ; Compare to table
            jl      End_TC2                 ; Jump if A/D < 1st table value
            jeq     End_TC1                 ; Jump if A/D = 1st table value
CMPloop2:   incd    R5                      ; Point to next value in table
            inc     R13                     ; Add one to temp value
            cmp     TC_Temps(R5),R14        ; Compare again
            jge     CMPloop2                ; If R14 greater or equal, loop

End_TC_lookup:
; Resolved to nearest degree, now multiply by 10 and interpolate tenths.
; After tenths are determined, simply add it.  Then will have temp to nearest
; tent in binary format.
            rla     R13                     ; rotate left
            mov     R13,R12                 ; copy
            rla     R12                     ; rotate the copy two more times
            rla     R12                     ;
            add     R12,R13                 ; then add.
            mov     R13,TC_TMP              ; Save off to RAM
            
; Interpolate to tenths. Tenths = ((ADCvalue-lower)x10)/(higher-lower)
            mov     TC_Temps(R5),R15        ; Move table value to register
            decd    R5                      ; decrement table pointer
            mov     TC_Temps(R5),R13        ; Now, the higher is in R15
                                            ; the ADC value is in R14
                                            ; and the lower is in R13
            sub     R13,R15                 ; Get delta between lower and higher
            sub     R13,R14                 ; Get delta between ADC value and lower
                                            ;             
; Multiply delta between ADC value and lower table value (whic is in R14) by 10
            rla     R14                     ; rotate left
            mov     R14,R12                 ; copy
            rla     R12                     ; rotate the copy two more times
            rla     R12                     ; then add.
            add     R12,R14                 ; Done wit x10.  Now need to divide.

; Divide (ADC-lower)x10 by delta between lower and upper to get tenths
;    Dividend = (ADC-lower)x10, whic is in R14
;    Divisor  = (higher-lower), whic is in R15
            mov     #0,R12                  ; Use R12 as counter for subtractions
            cmp     R15,R14                 ; Compare R14, R15
            jl      remain                  ; jump if R14<R15 - no division
Divide2:    sub     R15,R14                 ; Dividend-Divisor -> dividend
            inc.b   R12
            cmp     R15,R14                 ; Is Dividend>Divisor?
            jge     Divide2                 ; Yes, subtract and increment again
remain:     rla     R14                     ; Multiply remainder by two
            cmp     R15,R14                 ; Is Dividend>Divisor?
            jl      EndDiv2                 ; If not, return
            inc.b   R12                     ; If so, increment counter to round up

EndDiv2:    ;    Division now done, so Add tenths (R12) to the whole number (TC_TMP)
            add    R12,TC_TMP               ; Now have xx.x (in binary) in TC_TMP
            ret                             ; Done so return

End_TC1:    inc     R13                     ; Increment R13 to -40 deg C.
End_TC2:    mov     R13,TC_TMP              ; Move to RAM. 
            ret                             ; Return
            
;******************************************************************************
Get_ABS_temp:     ; Calculate absolute temperature by adding the TC and TR
;                 ; temps.
;                 ; 
;****************************************************************************** 

; Add TR_TMP and TC_TMP to get ABS_TMP 
Add_temps:  mov     TC_TMP,R14              ; Put TC_TMP in R14
            add     TR_TMP,R14              ; Add TR_TMP to it
            bit     #0x8000,R14             ; Test if result is negative
            jge     Bin_Dec                 ; If not, proceed wit conversion
            mov     R14,ABS_TMP             ; If so, store the negative temp
            jmp     End_Get_ABS_Temp        ; and jump to end
            
; Now convert to BCD format
Bin_Dec:    rla      R14                    ; Shift out upper nibble,
            rla      R14                    ; not used for this
            rla      R14                    ; conversion
            rla      R14                    ;
            mov      #0xC,R15               ; Loop counter
            clr      R12                    ; Result goes into R12
L3:         rla      R14                    ; Shift MSB into C
            dadd     R12,R12                ; Add R14 to itself, plus C
            dec      R15
            jnz      L3                     ; Jump if not done

; Done converting to BCD  
            mov      R12,ABS_TMP             ; Save absolute temp in BCD to RAM.

End_Get_ABS_Temp:
            eint                            ; re-enable interrupts
            ret
            
;******************************************************************************
DISPLCD:    ; Display temp on LCD.
;
;****************************************************************************** 
            mov      ABS_TMP,R14            ; Get temperature from RAM
            bit      #0x8000,R14            ; Test if temp is negative
            jn       Disp_L                 ; If so, display "low" on LCD
            cmp      #0x1000,R14            ; Test if temp >= 100
            jge      Disp_                 ; If so, display "high" on LCD

; display temp on LCD
            mov.b    R14,R11                ; Copy lower byte
            mov.b    R14,R12                ; copy lower byte
            rra.b    R12                    ; 
            rra.b    R12                    ; 
            rra.b    R12                    ; Rotate right to expose nibble
            rra.b    R12                    ;
            swpb     R14                    ; Swap hig and low bytes of data
            mov.b    R14,R13                ; Copy again
            and      #0xF,R13               ; 
            and      #0xF,R12               ; 
            and      #0xF,R11               ; 
            mov.b    LCD_Tab(R11),&LCDM1 
            mov.b    LCD_Tab(R12),&LCDM2 
            bis.b    #0x40,&LCDM2           ; Turn on decimal point          
            mov.b    LCD_Tab(R13),&LCDM3
            mov.b    #0,&LCDM4              ; Clear upper digit 
            ret

; Display "L" on LCD       
Disp_L:     mov.b    #0,&LCDM4
            mov.b    #0,&LCDM3
            mov.b    #0,&LCDM2
            mov.b    #d+e+f,&LCDM1
            ret

; Display "H" on LCD
Disp_:      mov.b    #0,&LCDM4
            mov.b    #0,&LCDM3
            mov.b    #0,&LCDM2
            mov.b    #b+c+e+f+g,&LCDM1                               
            ret
;******************************************************************************
TR_Temps:   .word   0x5DDE   ; 0C
            .word   0x5CE1   ; 1C
            .word   0x5BD5   ; 2C
            .word   0x5AB9   ; 3C
            .word   0x598A   ; 4C
            .word   0x5848   ; 5C
            .word   0x573D   ; 6C
            .word   0x5624   ; 7C
            .word   0x54FB   ; 8C
            .word   0x53C1   ; 9C
            .word   0x5274   ; 10C
            .word   0x515C   ; 11C
            .word   0x5035   ; 12C
            .word   0x4EFF   ; 13C
            .word   0x4DB9   ; 14C
            .word   0x4C62   ; 15C
            .word   0x4B46   ; 16C
            .word   0x4A1D   ; 17C
            .word   0x48E7   ; 18C
            .word   0x47A3   ; 19C
            .word   0x464F   ; 20C
            .word   0x4531   ; 21C
            .word   0x4408   ; 22C
            .word   0x42D3   ; 23C
            .word   0x4191   ; 24C
            .word   0x4042   ; 25C
            .word   0x3F28   ; 26C
            .word   0x3E05   ; 27C
            .word   0x3CD8   ; 28C
            .word   0x3BA0   ; 29C
            .word   0x3A5C   ; 30C
            .word   0x394B   ; 31C
            .word   0x3832   ; 32C
            .word   0x3711   ; 33C
            .word   0x35E6   ; 34C
            .word   0x34B1   ; 35C
            .word   0x33AF   ; 36C
            .word   0x32A5   ; 37C
            .word   0x3195   ; 38C
            .word   0x307C   ; 39C
            .word   0x2F5C   ; 40C
            .word   0x0000   ; Too High

;******************************************************************************

TC_Temps:   .word   0xFD78   ; -40C
            .word   0xFD87   ; -39C
            .word   0xFD97   ; -38C
            .word   0xFDA6   ; -37C
            .word   0xFDB6   ; -36C
            .word   0xFDC6   ; -35C
            .word   0xFDD6   ; -34C
            .word   0xFDE6   ; -33C
            .word   0xFDF5   ; -32C
            .word   0xFE05   ; -31C
            .word   0xFE15   ; -30C
            .word   0xFE25   ; -29C
            .word   0xFE35   ; -28C
            .word   0xFE45   ; -27C
            .word   0xFE55   ; -26C
            .word   0xFE65   ; -25C
            .word   0xFE75   ; -24C
            .word   0xFE85   ; -23C
            .word   0xFE95   ; -22C
            .word   0xFEA6   ; -21C
            .word   0xFEB6   ; -20C
            .word   0xFEC6   ; -19C
            .word   0xFED6   ; -18C
            .word   0xFEE7   ; -17C
            .word   0xFEF7   ; -16C
            .word   0xFF07   ; -15C
            .word   0xFF18   ; -14C
            .word   0xFF28   ; -13C
            .word   0xFF38   ; -12C
            .word   0xFF49   ; -11C
            .word   0xFF5A   ; -10C
            .word   0xFF6A   ; -9C
            .word   0xFF7B   ; -8C
            .word   0xFF8B   ; -7C
            .word   0xFF9C   ; -6C
            .word   0xFFAC   ; -5C
            .word   0xFFBD   ; -4C
            .word   0xFFCE   ; -3C
            .word   0xFFDE   ; -2C
            .word   0xFFEF   ; -1C
            .word   0x0000   ;  0C
            .word   0x0011   ;  1C
            .word   0x0022   ;  2C
            .word   0x0033   ;  3C
            .word   0x0043   ;  4C
            .word   0x0054   ;  5C
            .word   0x0065   ;  6C
            .word   0x0076   ;  7C
            .word   0x0087   ;  8C
            .word   0x0098   ;  9C
            .word   0x00A9   ; 10C
            .word   0x00BA   ; 11C
            .word   0x00CA   ; 12C
            .word   0x00DB   ; 13C
            .word   0x00EC   ; 14C
            .word   0x00FD   ; 15C
            .word   0x010E   ; 16C
            .word   0x011F   ; 17C
            .word   0x0131   ; 18C
            .word   0x0142   ; 19C
            .word   0x0153   ; 20C
            .word   0x0164   ; 21C
            .word   0x0175   ; 22C
            .word   0x0186   ; 23C
            .word   0x0198   ; 24C
            .word   0x01A9   ; 25C
            .word   0x01BA   ; 26C
            .word   0x01CB   ; 27C
            .word   0x01DC   ; 28C
            .word   0x01EE   ; 29C
            .word   0x01FF   ; 30C
            .word   0x0210   ; 31C
            .word   0x0222   ; 32C
            .word   0x0233   ; 33C
            .word   0x0244   ; 34C
            .word   0x0255   ; 35C
            .word   0x0267   ; 36C
            .word   0x0278   ; 37C
            .word   0x028A   ; 38C
            .word   0x029B   ; 39C
            .word   0x02AC   ; 40C
            .word   0x02BE   ; 41C
            .word   0x02CF   ; 42C
            .word   0x02E1   ; 43C
            .word   0x02F2   ; 44C
            .word   0x0303   ; 45C
            .word   0x0315   ; 46C
            .word   0x0326   ; 47C
            .word   0x0338   ; 48C
            .word   0x0349   ; 49C
            .word   0x035B   ; 50C
            .word   0x036C   ; 51C
            .word   0x037E   ; 52C
            .word   0x038F   ; 53C
            .word   0x03A1   ; 54C
            .word   0x03B3   ; 55C
            .word   0x03C4   ; 56C
            .word   0x03D5   ; 57C
            .word   0x03E7   ; 58C
            .word   0x03F9   ; 59C
            .word   0x040A   ; 60C
            .word   0x041C   ; 61C
            .word   0x042D   ; 62C
            .word   0x043F   ; 63C
            .word   0x0451   ; 64C
            .word   0x0462   ; 65C
            .word   0x0474   ; 66C
            .word   0x0486   ; 67C
            .word   0x0497   ; 68C
            .word   0x04A9   ; 69C
            .word   0x04BA   ; 70C
            .word   0x04CC   ; 71C
            .word   0x04DE   ; 72C
            .word   0x04EF   ; 73C
            .word   0x0501   ; 74C
            .word   0x0513   ; 75C
            .word   0x0524   ; 76C
            .word   0x0536   ; 77C
            .word   0x0548   ; 78C
            .word   0x0559   ; 79C
            .word   0x056B   ; 80C
            .word   0x057C   ; 81C
            .word   0x058E   ; 82C
            .word   0x05A0   ; 83C
            .word   0x05B1   ; 84C
            .word   0x05C3   ; 85C
            .word   0x05D5   ; 86C
            .word   0x05E6   ; 87C
            .word   0x05F8   ; 88C
            .word   0x0609   ; 89C
            .word   0x061B   ; 90C
            .word   0x062C   ; 91C
            .word   0x063E   ; 92C
            .word   0x0650   ; 93C
            .word   0x0662   ; 94C
            .word   0x0673   ; 95C
            .word   0x0685   ; 96C
            .word   0x0696   ; 97C
            .word   0x06A8   ; 98C
            .word   0x06B9   ; 99C
            .word   0x06CB   ; 100C
            .word   0x7FFF   ; Too high

;****************************************************************************** 
;  LCD Table of definitions
;  These use the bit definitions defined above
;  for eac segment to turn on the appropriate
;  segments to form each numeral
;******************************************************************************
LCD_Tab:    .byte    a+b+c+d+e+f            ; displays "0"
            .byte    b+c                    ; displays "1"
            .byte    a+b+d+e+g              ; displays "2"
            .byte    a+b+c+d+g              ; displays "3"
            .byte    b+c+f+g                ; displays "4"
            .byte    a+c+d+f+g              ; displays "5"
            .byte    a+c+d+e+f+g            ; displays "6"
            .byte    a+b+c                  ; displays "7"
            .byte    a+b+c+d+e+f+g          ; displays "8"
            .byte    a+b+c+d+f+g            ; displays "9"
            .byte    a+b+c+e+f+g            ; displays "A"
            .byte    c+d+e+f+g              ; displays "b"
            .byte    a+d+e+f                ; displays "C"
            .byte    b+c+d+e+g              ; displays "d"
            .byte    a+d+e+f+g              ; displays "E"
            .byte    a+e+f+g                ; displays "F"   

;****************************************************************************** 
P1_ISR:     ;Exit    LPM0
;******************************************************************************
            bic.b   #BIT0,&P1IFG        ; Clear P1.0 IFG
            bic.w   #LPM0,0(R1)         ; Clear LPM0 bits to exit LPM0 
            reti                        ; on return from interrupt	 

;****************************************************************************** 
; MSP430x41x Interrupt vectors             
;******************************************************************************
            .section    .vectors
            .org    PORT1_VECTOR,0xFF
            .word   P1_ISR              ; P1 ISR
            .org    RESET_VECTOR,0xFF
            .word   RESET               ; POR, ext. Reset, Watchdog
