; THIS PROGRAM IS PROVIDED "AS IS". TI MAKES NO WARRANTIES OR
; REPRESENTATIONS, EITHER EXPRESS, IMPLIED OR STATUTORY, 
; INCLUDING ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS 
; FOR A PARTICULAR PURPOSE, LACK OF VIRUSES, ACCURACY OR 
; COMPLETENESS OF RESPONSES, RESULTS AND LACK OF NEGLIGENCE. 
; TI DISCLAIMS ANY WARRANTY OF TITLE, QUIET ENJOYMENT, QUIET 
; POSSESSION, AND NON-INFRINGEMENT OF ANY THIRD PARTY 
; INTELLECTUAL PROPERTY RIGHTS WITH REGARD TO THE PROGRAM OR 
; YOUR USE OF THE PROGRAM.
;
; IN NO EVENT SHALL TI BE LIABLE FOR ANY SPECIAL, INCIDENTAL, 
; CONSEQUENTIAL OR INDIRECT DAMAGES, HOWEVER CAUSED, ON ANY 
; THEORY OF LIABILITY AND WHETHER OR NOT TI HAS BEEN ADVISED 
; OF THE POSSIBILITY OF SUCH DAMAGES, ARISING IN ANY WAY OUT 
; OF THIS AGREEMENT, THE PROGRAM, OR YOUR USE OF THE PROGRAM. 
; EXCLUDED DAMAGES INCLUDE, BUT ARE NOT LIMITED TO, COST OF 
; REMOVAL OR REINSTALLATION, COMPUTER TIME, LABOR COSTS, LOSS 
; OF GOODWILL, LOSS OF PROFITS, LOSS OF SAVINGS, OR LOSS OF 
; USE OR INTERRUPTION OF BUSINESS. IN NO EVENT WILL TI'S 
; AGGREGATE LIABILITY UNDER THIS AGREEMENT OR ARISING OUT OF 
; YOUR USE OF THE PROGRAM EXCEED FIVE HUNDRED DOLLARS 
; (U.S.$500).
;
; Unless otherwise stated, the Program written and copyrighted 
; by Texas Instruments is distributed as "freeware".  You may, 
; only under TI's copyright in the Program, use and modify the 
; Program without any charge or restriction.  You may 
; distribute to third parties, provided that you transfer a 
; copy of this license to the third party and the third party 
; agrees to these terms by its first use of the Program. You 
; must reproduce the copyright notice and any other legend of 
; ownership on each copy or partial copy, of the Program.
;
; You acknowledge and agree that the Program contains 
; copyrighted material, trade secrets and other TI proprietary 
; information and is protected by copyright laws, 
; international copyright treaties, and trade secret laws, as 
; well as other intellectual property laws.  To protect TI's 
; rights in the Program, you agree not to decompile, reverse 
; engineer, disassemble or otherwise translate any object code 
; versions of the Program to a human-readable form.  You agree 
; that in no event will you alter, remove or destroy any 
; copyright notice included in the Program.  TI reserves all 
; rights not specifically granted under this license. Except 
; as specifically provided herein, nothing in this agreement 
; shall be construed as conferring by implication, estoppel, 
; or otherwise, upon you, any license or other right under any 
; TI patents, copyrights or trade secrets.
;
; You may not use the Program in non-TI devices.
;************************************************************************
; MSP430F149 Demo - Parallel Communication with ADS8364 EVM
;
;             ADS8364          MSP430F149		  
;	   -------------      -------------
;         |        HOLDx|<---|P1.0         |
;         |             |    |             |
;CHA0(+)~>|IN+       RST|<---|P1.3         |
;CHA0(-)~>|IN+       /RD|<---|P1.2         |
;         |          CLK|<---|SMCLK        |
;CHA1(+)~>|IN+          |    |             |
;CHA1(-)~>|IN+       EOC|--->|P1.1         |
;         |             |    |             |
;CHB0(+)~>|IN+          |    |             |
;CHB0(-)~>|IN+    D[0.7}|<---|P2.0..P2.7   |
;         |             |    |             |
;CHB1(+)~>|IN+          |    |             |
;CHB1(-)~>|IN+          |
;         |             |      _VCC
;CHC0(+)~>|IN+          |      |
;CHC0(-)~>|IN+       A2 |------|
;         |          A1 |------|
;CHC0(+)~>|IN+          |
;CHC1(-)~>|IN+       A0 |------|
;         |          CS |------|
;                             DGND 
;
; Texas Instruments, Inc.
; Tom Hendrick
; Data Aquisition Applications - Dallas
; Apr. 2002
; Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;************************************************************************
#include   <signal.h>
#include   <io.h>
#include   "ADS8364.h"             // ADC Equations

;************************************************************************
;Constants
;************************************************************************
            .equ    READ,0x06         ; If BYTE = 0, 1x Channels
                                      ; If BYTE = 1, 2x Channels
                                      ; If ADD + BYTE = 1, 3x Channels
            .equ    TEST,0x01
            .equ    SAMPLES,0x480     ; Take 100 Samples From Each Channel
                                      ; 2Reads/Conversion x 6 Channels
                                      ; = 1200 Cycles
;************************************************************************
;Setup RAM
;************************************************************************
           .section .data
ADC_Data:  .DS   2                    ; Storage for ADC Samples

;************************************************************************
;Program Code                    
;************************************************************************
           .section .text
RESET_ISR: mov     #__stack,R1         ; define stackpointer
           call    #Init_Sys           ; Initialize the MSP430
           call    #RESET_ADC          ; Initialize the ADC
           call    #MAIN               ; Main Program

;************************************************************************
;Main Program Code                    
;************************************************************************
MAIN:

test_loop:
          mov.b #00, R8
          mov.b #00, R9  
          mov.b #00, R10
          mov.b #00, R11                 ; Set Data Pointer to zero
sample:
          bis.b #BIT0, R10               ; Set Read Test
          bic.b #HOLD,&P1OUT             ; Toggle HOLDx
          bis.b #HOLD,&P1OUT
wait:     bit.b #BIT0, R10               ; Interrupt Finished?
          jnz wait
          cmp.w #SAMPLES, R11            ; Finished sampling?
          jne sample
          jmp test_loop                  ; Start Over

;************************************************************************
Init_Sys:   ;Modules and Controls Registers set-up subroutine
;************************************************************************
StopWDT:  mov     #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer

SetupClock:
          bic.b #XT2OFF,&BCSCTL1
          bis.b #SELM1+SELS+DIVS0, &BCSCTL2
HF_WAIT:  ;8MHz Crystal used - wait for stabilization
          bic.b #OFIFG,&IFG1
          bit.b #OFIFG,&IFG1
          jnz HF_WAIT
          bic.b #OFIFG,&IFG1                  ; Clear Oscillator fault flag
          bit.b #OFIFG,&IFG1                  ; Test for clear
SetupPort1:
          clr.b &P1OUT
          bis.b #SMCLK+HOLD+RST+RE,&P1DIR	  ; P1 pin 1 set to output (toggle's LED)
          bis.b #HOLD+RST+RE,&P1OUT
          bis.b #SMCLK,&P1SEL	              ; P1.4 SMCLK option select
SetupPort2:
          clr.b &P2OUT
          bis.b #0,&P2DIR	       

SetupInterrupt:
          bic.b #0x2, &P1IFG           ; Clear interrupt flags
          bic.b #0x2, &P1IES           ; Set for edge selection
          bis.b #0x2, &P1IE            ; Enable external Interrupt
          eint                         ; Enable interrupts
          ret
          
;*********************************************************************************
RESET_ADC:  ;Initialize the AtoD Converter
;*********************************************************************************
          bic.b #RST,&P1OUT
          bis.b #RST,&P1OUT
          ret  

;*********************************************************************************
IRQ_ISR:
;*********************************************************************************
          bic.b #BIT0, R10
          mov.b #0, R12
loop:     bic.b #RE, &P1OUT
          mov.b &P3IN,R8 ; Store Lower Byte
          bis.b #RE, &P1OUT
          bic.b #RE, &P1OUT
          mov.b &P4IN,R9 ; Store Upper Byte
          bis.b #RE, &P1OUT           
          swpb R9
          add.w R8, R9
          mov.w R9, ADC_Data(R11)
          incd R11
          inc R12
          cmp.b #READ, R12            ; Finished reading?
          jne loop     
          bic.b #0x2,&P1IFG
          reti                        ; return from interrupt
           
;*********************************************************************************
;         COMMON  INTVEC              ; MSP430x11x1/MSP430F14x Interrupt vectors   
;*********************************************************************************
            .section    .vectors
            .org    PORT1_VECTOR,0xFF
PORT1_VEC:  .word   IRQ_ISR             ; PORT1, Ext. Int.
            .org    RESET_VECTOR,0xFF
RESET_VEC:  .word   RESET_ISR           ; POR, ext. Reset, Watchdog
