#include <io.h>
;******************************************************************************
;   MSP-FET430x110 Demo - BasicClock Output buffered SMCLK, ACLK and MCLK/12 
;     
;   Description; Buffer ACLK on P2.0, default SMCLK(DCO) on P1.4 and MCLK/12 on 
;   P1.1.  To measure SMCLK on P1.4 with the FET, JTAG must be released, and 
;   the debugger stopped and re-started to clear breakpoint logic.
;   ACLK = LFXT1 = 32768, MCLK = SMCLK = default DCO ~ 800k
;   //*External watch crystal installed on XIN XOUT is required for ACLK*//	  
;
;                MSP430F1121
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32kHz
;          --|RST          XOUT|-
;            |                 |
;            |             P1.4|-->SMCLK~ 800kHz
;            |             P1.1|-->MCLK/12 = DCO/12
;            |             P2.0|-->ACLK = 32kHz
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
            bis.b   #0x12,&P1DIR            ; P1.1,4 output direction
            bis.b   #0x10,&P1SEL            ; P1.4 = SMCLK
            bis.b   #0x01,&P2DIR            ; P2.0 output direction
            bis.b   #0x01,&P2SEL            ; P2.0 = ACLK
                                            ;
Mainloop:   xor.b   #0x02,&P1OUT            ; P1.1 = Toggle
            jmp     Mainloop                ; Repeat
                                            ;           
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x11x1         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
