#include <io.h>
;*****************************************************************************
;   MSP-FET430x110 Demo - Software SPI Interface with HC165 Shift Register
;
;   Description: Read an 'HC165 shift register.  /LD is pulsed low to latch 
;   in parellel data, then 8-bits are shifted out of the HC165 into the MSP430 
;   stored in register Data (R4). 
;   ACLK = n/a, MCLK = SMCLK = default DCO ~ 800k 
;
;                       MSP430F1121
;                    -----------------
;                /|\|              XIN|-  
;                 | |                 | 
;       HC165     --|RST          XOUT|-
;     ----------    |                 |
;    |      /LD|<---|P2.0             |    
;  ->|A-H   CLK|<---|P2.1             |
;  |-|INH    QH|--->|P2.2             |    
;  |-|SER      |    |                 | 
;  v
;
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;*****************************************************************************
#define     Data    R4                  
            .equ    LD,1                    ; P2.0 LD
            .equ    CLK,2                   ; P2.1 Serial clock
            .equ    QH,4                    ; P2.2 Data out

            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize 'x112x stack
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop Watchdog Timer
SetupP2:    mov.b   #LD,&P2OUT              ; /LD set, - P2.x reset
            bis.b   #LD+CLK,&P2DIR          ; /LD and CLK outputs
                                            ;
Mainloop:   call    #Read_HC165             ; Data <-- HC165
            jmp     Mainloop                ; Repeat
                                            ;
;----------------------------------------------------------------------------- 
Read_HC165: ;Shift data from HC165 --> Data (R4)     
;----------------------------------------------------------------------------- 
            bic.b   #LD,&P2OUT              ; /LD reset 
            bis.b   #LD,&P2OUT              ; /LD set 
            clr.w   Data                    ;
            push.b  #8                      ; 8 bits to Read - TOS  
RX_loop:    bit.b   #QH,&P2IN               ; QH --> Carry
            rlc.b   Data                    ; Carry --> Data
            bis.b   #CLK,&P2OUT             ; CLK set 
            bic.b   #CLK,&P2OUT             ; CLK reset
            dec.b   0(R1)                   ; All 8 bits sent?
            jnz     RX_loop                 ;
            incd.w  R1                      ; bit counter off stack
            ret                             ; Return from subroutine
                                            ;
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430x11x1         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
