#include <io.h>
;******************************************************************************
;   MSP-FET430x110 Demo - BasicClock LFXT1/MCLK configured with HF XTAL
;
;   Description; Proper selection of an external HF XTAL for MCLK is shown 
;   by first polling the OSC fault until XTAL is stable - only then is MCLK
;   sourced by LFXT1.  MCLK/10 is on P1.1 driven by a software loop taking 
;   exactly 10 CPU cycles. 
;   ACLK = MCLK = LFXT1 = HFXTAL, SMCLK = DCO  
;   //** HF XTAL NOT INSTALLED ON FET **//
;
;                MSP430F1121
;             -----------------
;         /|\|              XIN|-  
;          | |                 | HF XTAL (455k - 8Mhz) 
;          --|RST          XOUT|-
;            |                 |
;            |             P1.1|-->MCLK/10 = HFXTAL/10
;            |             P2.0|-->ACLK = HFXTAL
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupBC:    bis.b   #XTS,&BCSCTL1           ; LFXT1 = HF XTAL
SetupOsc:   bic.b   #OFIFG,&IFG1            ; Clear OSC fault flag 
            mov.w   #0xFF,R15               ; R15 = Delay 
SetupOsc1:  dec.w   R15                     ; Additional delay to ensure start 
            jnz     SetupOsc1               ;
            bit.b   #OFIFG,&IFG1            ; OSC fault flag set? 
            jnz     SetupOsc                ; OSC Fault, clear flag again 
            bis.b   #SELM1+SELM0,&BCSCTL2   ; MCLK = LFXT1
                                            ;           
            bis.b   #0x01,&P2DIR            ; P2.0 = output direction
            bis.b   #0x01,&P2SEL            ; P2.0 = ACLK function
            bis.b   #0x02,&P1DIR            ; P1.1 = output direction
                                            ;           
Mainloop:   bis.b   #0x02,&P1OUT            ; P1.1 = 1
            bic.b   #0x02,&P1OUT            ; P1.1 = 0
            jmp     Mainloop                ; Repeat
                                            ;           
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x11x1         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
