#include <io.h>
;******************************************************************************
;    MSP-FET430P120 Demo - ADC10 Sample A0, AVcc 
;
;   Description; A single sample is made on A0 with reference to Avcc.  
;   Software sets ADC10SC to start sample and conversion - ADC10SC 
;   automatically cleared at EOC. ADC10 internal oscillator times sample (16x)
;   and conversion. ADC10BUSY flag is polled for EOC. Conversion code in 
;   ADC10MEM. 
;   //* MSP430F12x2 or MSP430F11x2 Device Required*//
;
;                MSP430F12x2
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;        >---|A0               |
;
;   M.Buccini
;   Texas Instruments, Inc
;   February 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC10: mov.w   #ADC10ON,&ADC10CTL0     ;
            bis.b   #0x01,&ADC10AE          ; P2.0 ADC10 option select 
                                            ;
Mainloop:   bis.w   #ENC+ADC10SC,&ADC10CTL0 ; Start sampling
L1:         bit.w   #ADC10BUSY,&ADC10CTL1   ; ADC10 BUSY?
            jnz     L1                      ;        
            jmp     Mainloop                ; Again
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x2         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
