#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - ADC10 Sample A10 Temp Set P1.0 if temp > ~ 29c 
;
;   Description; A single sample is made on A10 with reference to internal
;   1.5V Vref. Software sets ADC10SC to start sample and conversion - ADC10SC 
;   automatically cleared at EOC. (ADC10OSC/4)/64 determines sample time which
;   needs to be greater than 30us for temperature sensor. In Mainloop MSP430 
;   waits in LPM0 to save power until ADC10 conversion complete, ADC10_ISR will
;   force exit from any LPMx in Mainloop on reti. If A10 > 1088mv, P1.0 set, 
;   else reset. A10 ~ 1088mv or 0x2E7 with 1.5v reference very approximate 29c.
;   Temperature sensor offset and slope will vary from device to device per 
;   datasheet tolerence. 
;   //* MSP430F1232 or MSP430F1132 Device Required*//
;
;                MSP430F1232
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |A10          P1.0|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC10: mov.w   #INCH_10+ADC10DIV_3,&ADC10CTL1 ; Temp Sensor ADC10CLK/4    
            mov.w   #SREF_1+ADC10SHT_3+REFON+ADC10ON+ADC10IE,&ADC10CTL0 ; 
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output
                                            ;
Mainloop:   bis.w   #ENC+ADC10SC,&ADC10CTL0 ; Start sampling/conversion
            bis.w   #CPUOFF+GIE,R2          ; Hold, ADC12 ISR will force exit
            bic.b   #0x01,&P1OUT            ; P1.0 = 0
            cmp.w   #0x2E7,&ADC10MEM        ; ADC10MEM = A10 > 0x2E7?
            jlo     Mainloop                ; Again
            bis.b   #0x01,&P1OUT            ; P1.0 = 1
            jmp     Mainloop                ;
                                            ;
;------------------------------------------------------------------------------ 
ADC10_ISR:
;------------------------------------------------------------------------------ 
            bic.w   #CPUOFF,0(R1)           ; Exit LPM0 on reti
            reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x2         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x0A,0xFF               ; ADC10 Vector
            .word   ADC10_ISR               ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
