#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - ADC10 Sample A11 lo_Batt Set P1.0 if AVcc < 2.3V 
;
;   Description; A single sample is made on A11 (AVcc/2) with reference to 
;   internal 1.5V Vref. Software sets ADC10SC to start sample and conversion 
;   - ADC10SC automatically cleared at EOC. ADC10 internal oscillator times 
;   sample (16x) and conversion. ADC10BUSY flag is polled for EOC. If A11 
;  (AVcc/2) < 0x311 (0.65V) indicating AVcc is less 2.3V, P1.0 set indicating 
;   a lo_Batt condition, else reset. 
;   //*MSP430F1232 or MSP430F1132 Device Required*//
;
;                MSP430F1232
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |A11         P1.0 |--> LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC10: mov.w   #INCH_11,&ADC10CTL1     ; AVcc/2    
            mov.w   #SREF_1+ADC10SHT_2+REFON+ADC10ON,&ADC10CTL0; 
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output
                                            ;          
Mainloop:   bis.w   #ENC+ADC10SC,&ADC10CTL0 ; Start sampling/conversion
L1:         bit.w   #ADC10BUSY,&ADC10CTL1   ; ADC10BUSY?
            jnz     L1                      ;        
            cmp.w   #0x311,&ADC10MEM        ; ADC10MEM = A11 (AVcc/2) > 0.65V?
            jlo     lo_Batt                 ;
            bic.b   #0x01,&P1OUT            ; P1.0 = 0
            jmp     Mainloop                ; Again
lo_Batt:    bis.b   #0x01,&P1OUT            ; P1.0 = 1
            jmp     Mainloop                ; Again
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x2         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
