#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - ADC10 Sample A0 64x, AVcc, Repeat Single DTC DCO 
;
;   Description; Use DTC to sample A0 64 times with reference to AVcc. Software  
;   writes to ADC10SC to trigger sample burst. In Mainloop MSP430 waits in LPM0
;   to save power until ADC10 conversion burst complete, ADC10_ISR(DTC) will 
;   force exit from any LPMx in Mainloop on reti. ADC10 internal oscillator 
;   times sample period (16x) and conversion (13x). DTC transfers conversion 
;   code to RAM 0x200 - 0x280. P1.0 set at start of conversion burst, reset on
;   completion. 
;   //*MSP430F1232 or MSP430F1132 Device Required *//
;
;                MSP430F1232
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;        >---|A0           P1.0|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC10: mov.w   #CONSEQ_2,&ADC10CTL1    ; Repeat single channel
            mov.w   #ADC10SHT_2+MSC+ADC10ON+ADC10IE,&ADC10CTL0 ; 
            bis.b   #0x01,&ADC10AE          ; P2.0 ADC option select 
            mov.b   #0x40,&ADC10DTC1        ; 64 conversions   
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output
                                            ;
Mainloop:   bic.w   #ENC,&ADC10CTL0         ; 
busy_test:  bit     #BUSY,&ADC10CTL1        ; ADC10 core must be inactive
            jnz     busy_test               ;
            mov.w   #0x200,&ADC10SA         ; Data buffer start
            bis.b   #0x01,&P1OUT            ; P1.0 = 1 
            bis.w   #ENC+ADC10SC,&ADC10CTL0 ; Sampling and conversion start
            bis.w   #CPUOFF+GIE,R2          ; LPM0, ADC12 ISR will force exit
            bic.b   #0x01,&P1OUT            ; P1.0 = 0 
            jmp     Mainloop                ; Again
                                            ;
;------------------------------------------------------------------------------ 
ADC10_ISR:   
;------------------------------------------------------------------------------ 
            mov.w   #GIE,0(R1)              ; Exit any LPMx on reti
            reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x2         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .ORG    0x0A,0xFF               ; ADC10 Vector
            .WORD   ADC10_ISR               ; 
            .ORG    0x1E,0xFF               ; MSP430 RESET Vector
            .WORD   RESET                   ; 
