#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - ADC10 Sample A0, 1.5V, TA1 Trigger Set P1.0 if > 0.5V  
;
;   Description; A0 is sampled 16/second (2048ACLK)with reference to 1.5V. 
;   Timer_A is run in upmode and TA1 is used to automatically trigger
;   ADC10 conversion, TA0 defines the period. Internal oscillator times sample
;   (16x) and conversion (13x). Inside ADC10_ISR if A0 > 0.5Vcc, P1.0 is set, 
;   else reset. Normal mode is LPM3.
;   //*An external watch crystal on XIN XOUT is required for ACLK*//	  
;   //*MSP430F1232 or MSP430F1132 Device Required*//
;
;                MSP430F1232
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32kHz
;          --|RST          XOUT|-
;            |                 |
;        >---|A0          P1.0 |--> LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   February 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC10: mov.w   #SHS_1+CONSEQ_2,&ADC10CTL1 ; TA1 trigger sample start
            mov.w   #SREF_1+ADC10SHT_2+REFON+ADC10ON+ADC10IE,&ADC10CTL0; 
            bis.w   #ENC,&ADC10CTL0         ; ADC10 enable set
            bis.b   #0x01,&ADC10AE          ; P2.0 ADC10 option select 
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output
SetupC0:    mov.w   #2048-1,&CCR0           ; PWM Period 
SetupC1:    mov.w   #OUTMOD1+OUTMOD0,&CCTL1 ; CCR1 set/reset
            mov.w   #2047,&CCR1             ; CCR1 PWM Duty Cycle	  
SetupTA:    mov.w   #TASSEL0+MC0,TACTL      ; ACLK, up mode
                                            ;
Mainloop:   bis.w   #LPM3+GIE,R2            ; Enter LPM3
            nop                             ; Required only for C-spy
                                            ;
;------------------------------------------------------------------------------ 
ADC10_ISR:  
;------------------------------------------------------------------------------ 
            bic.b   #0x01,&P1OUT            ; P1.0 = 0
            cmp.w   #0x155,&ADC10MEM        ; ADC10MEM = A0 > 0.5V?
            jlo     ADC10_Exit              ; Again
            bis.b   #0x01,&P1OUT            ; P1.0 = 1
ADC10_Exit: reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x2         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x0A,0xFF               ; ADC10 Vector
            .word   ADC10_ISR               ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
