#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - DCOCLK Biased with External Resistor Rosc
;     
;   Description; Use external resistor on P2.5 as DCO current source. DCOCLK 
;   is used by default as the MCLK source. Mainloop drives a 10 cycle software 
;   loop toggling P1.1. Measure DCOCLK/10 on P1.1. Example also disables XTAL 
;   buffer. Use of external resistor reduces temperature sensitivity of DCOCLK.
;
;   DCOCLK = Using default DCO and RSEL values @ 2.85V
;   022k ~ 5.5MHz
;   100k ~ 2Mhz
;   220k ~ 860kHz
;   470k ~ 400kHz
;
;               MSP430F123(2)
;             -----------------
;    /|\  /|\|              XIN|-  
;     |    | |                 | 
;     R    --|RST          XOUT|-
;     |      |                 |
;     -------|P2.5/ROSC    P1.1|-->DCOCLK/10
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupBC:    bis.b   #DCOR,&BCSCTL2          ; Rosc 
            bis.w   #OSCOFF,R2              ; XTAL not used
            bis.b   #0x02,&P1DIR            ; P1.1 = output direction
                                            ;           
Mainloop:   bis.b   #0x02,&P1OUT            ; P1.1 = 1
            bic.b   #0x02,&P1OUT            ; P1.1 = 0
            jmp     Mainloop                ; Repeat
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x12x(2)         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
