#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - Timer_A Toggle P1.0, overflow ISR, 32kHz ACLK 
;
;   Description; Toggle P1.0 using using software and the timer_A overflow ISR. 
;   In this example an ISR will trigger when TA overflows.  Inside the ISR P1.0 
;   is toggled. Toggle rate is exactly 0.25hz.
;   ACLK = TACLK = 32768, MCLK = DCO ~ 800k
;   //*An external watch crystal on XIN XOUT is required for ACLK*//	  
;
;               MSP430F123(2)
;             -----------------
;         /|\|              XIN|-   
;          | |                 | 32k 
;          --|RST          XOUT|-
;            |                 |
;            |             P1.0|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupTA:    mov.w   #TASSEL0+TACLR+TAIE,&TACTL  ; ACLK, Clr TAR, interrupt
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output 
            bis.w   #MC1,&TACTL             ; Start timer_A continous mode
            eint                            ; Enable interrupts
                                            ;													  
Mainloop:   bis.w   #LPM3,R2                ; MCLK is not required
            nop                             ; Required only for C-spy
                                            ;
;------------------------------------------------------------------------------ 
TAX_ISR:    ;Common ISR for CCR1-4 and overflow
;------------------------------------------------------------------------------ 
            add.w   &TAIV,R0                ; Add Timer_A offset vector
            reti                            ; CCR0 - no source
            reti                            ; CCR1
            reti                            ; CCR2
            reti                            ; CCR1
            reti                            ; CCR4
TA_over:    xor.b   #0x01,&P1OUT            ; Toggle P1.0
            reti                            ; Return from overflow ISR		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430F12x          
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x10,0xFF               ; Timer_AX Vector
            .word   TAX_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
