#include <io.h>
;******************************************************************************
;   MSP-FET430P120 Demo - Timer_A Toggle P1.1 With TA0 up-downmode, DCO SMCLK 
;
;   Description; Toggle P1.1 using hardware TA0 output. Timer_A is configured
;   in up-downmode with CCR0 defining period, TA0 also output on P1.1. In this 
;   example, CCR0 is loaded with 250 and TA0 will toggling P.1 at TACLK/2*250.
;   Thus the ouput frequency on P1.1 will be the TACLK/1000. No CPU or software 
;   resources required.
;   SMCLK = MCLK = TACLK = default DCO ~ 800kHz
;   As coded with TACLK = SMCLK, P1.1 output frequency ~ 800000/1000
;
;               MSP430F123(2)
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |             P1.1|-->TA0 SMCLK/1000
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupTA:    mov.w   #TASSEL1+TACLR,&TACTL   ; SMCLK, clear TAR
SetupC0:    mov.w   #OUTMOD_4,&CCTL0        ; CCR0 toggle mode
            mov.w   #250,&CCR0              ; 
SetupP1:    bis.b   #0x02,&P1DIR            ; P1.1 output 
            bis.b   #0x02,&P1SEL            ; P1.1 option slect 
            bis.w   #MC1+MC0,&TACTL         ; Start Timer_a in up-downmode
                                            ;						  
Mainloop:   bis.w   #CPUOFF,R2              ; CPU off
            nop                             ; Required for C-spy
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x11x1         
;------------------------------------------------------------------------------
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
