//******************************************************************************
//  MSP-FET430P120 Demo - ADC10 Sample A10 64x, 1.5v, Repeat Single DTC/DCO
//
//  Description; Use DTC to sample A10 64 times with reference to internal 1.5v.
//  Vref Software writes to ADC10SC to trigger sample burst. In Mainloop MSP430 
//  waits in LPM0 to save power until ADC10 conversion complete, ADC10_ISR(DTC)
//  will force exit from any LPMx in Mainloop on reti. (ADC10OSC/4)/64 
//  determines sample time which needs to be greater than 30us for temperature
//  sensor. DTC transfers conversion code to RAM 200h - 280h. P1.0 set at start
//  of conversion burst, reset oncompletion. Temperature sensor offset and slope
//  will vary from device to device per datasheet tolerence. 
//  //*MSP430F1232 or MSP430F1132 Device Required*//
//
//                MSP430F1232
//             -----------------
//         /|\|              XIN|-  
//          | |                 |
//          --|RST          XOUT|-
//            |                 |
//            |A10           P1.0|-->LED
//
//  M.Buccini
//  Texas Instruments, Inc
//  January 2002
//  Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
//******************************************************************************

#include  <signal.h>
#include  <io.h>

int main(void)
{ 
  WDTCTL = WDTPW + WDTHOLD;             // Stop WDT
  ADC10CTL1 = INCH_10 + ADC10DIV_3 + CONSEQ_2;       
  ADC10CTL0 = SREF_1 + ADC10SHT_3 + MSC+REFON + ADC10ON + ADC10IE;
  ADC10DTC1 = 0x40;                     // 64 conversions   
  ADC10AE |= 0x01;                      // P2.0 ADC option select 
  P1DIR |= 0x01;                        // Set P1.0 output
  
  for (;;)                              
  {
    ADC10CTL0 &= ~ENC;                  // 
    while ((ADC10CTL1 & BUSY) == BUSY); // ADC10 core must be inactive
    ADC10SA = 0x200;                    // Data buffer start
    P1OUT |= 0x01;                      // P1.0 = 1
    ADC10CTL0 |= ENC + ADC10SC;         // Sampling and conversion start
    _BIS_SR(CPUOFF + GIE);              // LPM0, ADC12 ISR will force exit
    P1OUT &= ~0x01;                     // P1.0 = 0
  }
  return 0;
}

// ADC10 interrupt service routine
interrupt (ADC10_VECTOR) ADC10_ISR(void)
{
  _BIC_SR_IRQ(CPUOFF);                // Clear CPUOFF bit from 0(SR)
}
