#include <io.h>
;******************************************************************************
;   MSP-FET430P140 Demo - Timer_A Toggle P1.0, CCR0 upmode ISR, 32kHz ACLK 
;
;   Description; Toggle P1.0 using software and the TA_0 ISR. Timer_A is 
;   configured in an upmode, thus the the timer will overflow when TAR counts 
;   to CCR0. In this example, CCR0 is loaded with 1000.
;   ACLK = TACLK = LFXT1 = 32768, MCLK = SMCLK = DCO ~ 800k
;   //*An external watch crystal on XIN XOUT is required for ACLK*//	  
;
;                MSP430F149
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32kHz 
;          --|RST          XOUT|-
;            |                 |
;            |             P1.0|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize 'F149 stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupTA:    mov.w   #TASSEL0+TACLR,&TACTL   ; ACLK, clear TAR
SetupC0:    mov.w   #CCIE,&CCTL0            ; CCR0 interrupt enabled
            mov.w   #1000-1,&CCR0           ; CCR0 counts to 1000
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output 
            bis.w   #MC0,&TACTL             ; Start Timer_a in upmode
            eint                            ; Enable interrupts
                                            ;													  
Mainloop:   bis.w   #LPM3,R2                ; Enter LPM3
            nop                             ; Required for C-spy
                                            ;
;------------------------------------------------------------------------------ 
TA0_ISR:    ;Toggle P1.0
;------------------------------------------------------------------------------ 
            xor.b   #0x01,&P1OUT            ; Toggle P1.0
            reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x13x/14x          
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x0C,0xFF               ; Timer_A0 Vector
            .word   TA0_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
