#include <io.h>
;******************************************************************************
;   MSP-FET430P140 Demo - Timer_B Toggle P1.0, TBCCR0 Contmode ISR, DCO SMCLK
;
;   Description; Toggle P1.0 using using software and the TB_0 ISR. Toggle rate
;   is set at 50000 DCO/SMCLK cycles. Default DCO frequency is used for TBCLK.  
;   Durring the TB_0 ISR P0.1 is toggled and 50000 additional cycles are added 
;   to TBCCR0.  TB_0 ISR is triggered every 50000 cycles. CPU us used only 
;   durring TB_ISR.
;   MCLK = SMCLK = TBCLK = DCO ~ 800k 
;
;                 MSP430F149
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |             P1.0|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize 'F149 stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupTB:    mov.w   #TBSSEL1+TBCLR,&TBCTL   ; SMCLK, clear TAR
SetupC0:    mov.w   #CCIE,&TBCCTL0          ; CCR0 interrupt enabled
            mov.w   #50000,&TBCCR0          ; 
SetupP1:    bis.b   #0x01,&P1DIR            ; P1.0 output 
            bis.w   #MC1,&TBCTL             ; Start Timer_a in continous mode
            eint                            ; Enable interrupts
                                            ;						  
Mainloop:   bis.w   #CPUOFF,R2              ; CPU off
            nop                             ; Required for C-spy
                                            ;
;------------------------------------------------------------------------------ 
TB0_ISR:    ;Toggle P1.0
;------------------------------------------------------------------------------ 
            xor.b   #0x01,&P1OUT            ; Toggle P1.0
            add.w   #50000,&TBCCR0          ; Add Offset to CCR0
            reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x13x/14x          
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1A,0xFF               ; Timer_B0 Vector
            .word   TB0_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
