#include <io.h>
;******************************************************************************
;   MSP-FET430P140 Demo - USART0 UART 2400 Ultra-low Power Echo ISR, 32kHz ACLK
;
;   Description; Echo a received character, RX ISR used. Normal mode is LPM3, 
;   USART0 RX interrupt triggers TX Echo. 
;   ACLK = UCLK0 = LFXT1 = 32768Hz, MCLK = SMCLK = DCOCLK
;   Baud rate divider with 32768hz XTAL @2400 = 32768Hz/2400 = 13.65 (0x00D)
;   //*An external watch crystal on XIN XOUT is required for ACLK*//	  
;              
;                 MSP430F149
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32kHz  
;          --|RST          XOUT|-
;            |                 |
;            |             P3.4|-----------> 
;            |                 | 2400 - 8N1
;            |             P3.5|<-----------
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize 'F149 stackpointer
            call    #Init_Sys               ;
                                            ;   
Mainloop:   bis.b   #LPM3,R2                ; Enter LPM3
            jmp     Mainloop                ; Do nothing
                                            ;
;------------------------------------------------------------------------------ 
Init_Sys:   ;Initalize MSP430 system
;------------------------------------------------------------------------------ 
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupUART0: mov.b   #CHAR,&UCTL0            ; 8-bit characters
            mov.b   #SSEL0,&UTCTL0          ; UCLK = ACLK
            mov.b   #0x0D,&UBR00            ; 32k/2400 - 13.65
            mov.b   #0x00,&UBR10            ; 32k 2400
            mov.b   #0x6B,&UMCTL0           ; 32k 2400 modulation 
            bis.b   #UTXE0+URXE0,&ME1       ; Enable USART0 TXD/RXD
            bis.b   #URXIE0,&IE1            ; Enable USART0 RX interrupt
SetupP3:    bis.b   #0x30,&P3SEL            ; P3.4,5 = USART0 TXD/RXD
            bis.b   #0x10,&P3DIR            ; P3.4 = output direction 
            eint                            ; General enable interrupts
            ret                             ; Return from subroutine
                                            ;             
;------------------------------------------------------------------------------ 
USART_ISR:  ;Echo back RXed character, confirm TX buffer is ready first
;------------------------------------------------------------------------------ 
TX1:        bit.b   #UTXIFG0,&IFG1          ; USART0 TX buffer ready?
            jz      TX1                     ; Jump is TX buffer not ready
            mov.b   &RXBUF0,&TXBUF0         ; TX -> RXed character
            reti                            ; 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x13x/14x        
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x12,0xFF               ; 
            .word   USART_ISR               ; USART0 receive
            .org    0x1E,0xFF               ; 
            .word   RESET                   ; POR, ext. Reset, Watchdog
