#include <io.h>
;******************************************************************************
;   MSP-FET430P410 Demo - BasicTimer Toggle P5.1 using ISR, 32kHz ACLK 
;
;   Description; This program will toggle P5.1 by xor'ing P5.1 inside of
;   a basic timer ISR.  ACLK used to clock basic timer. 
;   ACLK = LFXT1 = 32768, MCLK = SMCLK = DCO
;   //*An external watch crystal on XIN XOUT is required for ACLK*//	  
;
;                MSP430F413
;             -----------------
;         /|\|              XIN|-  
;          | |                 | 32k
;          --|RST          XOUT|-
;            |                 |
;            |             P5.1|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0x300,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupFLL:   bis.b   #XCAP14PF,&FLL_CTL0     ; Configure load caps
SetupBT:    mov.b   #BTDIV+BTIP1+BTIP0,&BTCTL   ; 125ms Int. 
            bis.b   #BTIE,&IE2              ; Enable Basic Timer interrupt        
SetupP5:    bis.b   #0x02,&P5DIR            ; P5.1 output
            eint                            ; Enable interrupts      
                                            ;				  
Mainloop:   bis.w   #LPM3,R2                ; Remain in LPM3
            nop                             ; Required only for C-spy
                                            ;
;------------------------------------------------------------------------------
BT_ISR:     ;Toggle P5.1
;------------------------------------------------------------------------------
            xor.b   #0x02,&P5OUT            ; Toggle P5.1
            reti                            ;		 
                                            ;   
;----------------------------------------------------------------------------- 
;           Interrupt Vectors Used MSP430x41x         
;----------------------------------------------------------------------------- 
            .section    .vectors
            .org    0x00,0xFF               ; BT Vector
            .word   BT_ISR                  ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
