//******************************************************************************
//  MSP-FET430P410 Demo - Timer_A Toggle P5.1, overflow ISR, DCO SMCLK 
//
//  Description; This program will toggle P5.1 using software and the timer_A 
//  overflow ISR. In this example an ISR will trigger when TA overflows.  
//  Inside the ISR P5.1 is toggled.  Toggle rate is 16hz using DCO frequency 
//  of 1048576 with default FLL and watch crystal.	 
//  ACLK = LFXT1 =32768, MCLK = SMCLK = TACLK = DCO = ACLKx32 =1.048576MHz
//  Proper use of TAIV interrupt vector generator demonstrated.
//  //*An external watch crystal on XIN XOUT is required for ACLK*//	  
//  
//           MSP430F413
//         ---------------
//     /|\|            XIN|-  
//      | |               | 32kHz
//      --|RST        XOUT|-
//        |               |
//        |           P5.1|-->LED
//
//  M. Buccini
//  Texas Instruments, Inc
//  January 2002
//  Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
//******************************************************************************

#include  <signal.h>
#include  <io.h>

int main(void)
{ 
  WDTCTL = WDTPW + WDTHOLD;             // Stop WDT
  FLL_CTL0 |= XCAP14PF;                 // Configure load caps
  TACTL = TASSEL1 + TACLR + TAIE;       // SMCLK, clr. TAR, interrupt
  P5DIR |= 0x02;                        // Set P5.1 to output direction
  TACTL |= MC1;                         // Start Timer_a in continous
  _EINT();                              // Enable interrupts
 
  for (;;)                              
  {
    _BIS_SR(CPUOFF);                    // CPU off
    _NOP();                             // Required only for C-spy
  }
  return 0;
}

// Timer_A3 Interrupt Vector (TAIV) handler
interrupt (TIMERA1_VECTOR) Timer_A(void)
{
  switch( TAIV )
  {
   case  2: break;                      // CCR1 not used
   case  4: break;                      // CCR2 not used
   case 10: P5OUT ^= 0x02;              // overflow
            break;
  }
}
