#include <io.h>
;******************************************************************************
;   MSP-FET430P440 Demo - ADC12 Sample A0, AVcc
;
;   Description; A simple single sample is made on A0 with refernce to Avcc.  
;   Software controls pulse on ADC12SC to sample analog input signal and start 
;   conversion. ADC12  internal oscillator times conversion. ADC12BUSY flag is 
;   polled for EOC. Conversion result saved in ADC12MEM0. 
;    // For accurate conversion, bias ADC12 references per device datasheet //  
;  
;
;                MSP430F449
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;        >---|A0               |
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupADC12: mov.w   #ADC12ON,&ADC12CTL0     ; ADC12ON
            bis.b   #0x01,&P6SEL            ; P6.0 ADC option select 
                                            ;
Mainloop:   bis.w   #ENC+ADC12SC,&ADC12CTL0 ; Sampling open
            bic.w   #ADC12SC,&ADC12CTL0     ; Sampling closed, start conversion
L1:         bit.w   #ADC12BUSY,&ADC12CTL1   ; ADC12BUSY?
            jnz     L1                      ;        
            jmp     Mainloop                ; Again
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x44x         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
