#include <io.h>
;******************************************************************************
;   MSP-FET430P440 Demo - ADC12, Repeated Single Channel Conversions
;
;   Description: This program will show how to perform repeated conversions
;                on a single channel, using the "repeat-single-channel" mode.
;
;   This example shows how to perform repeated conversions on a single channel
;   using "repeat-single-channel" mode.  AVcc is used for the reference and 
;   repeated conversions are performed on Channel A0. Each conversion result 
;   is stored in ADC12MEM0 and is moved to a RAM location.  The conversion 
;   results are moved to RAM addresses 0x200 - 0x20E.  Test by applying a 
;   voltage to channel A0, then running. To view the conversion results, open a
;   memory window in C-Spy and view the contents of addresses 0x200 - 0x20E 
;   after stopping program execution. Remember the conversion results are 
;   12-bits so you must set the memory window to 16-bit mode.
;
;
;                MSP430F449
;              ---------------
;             |               |
;             |      A0 (P6.0)|<---- Vin
;             |               |
;
;
;   M.Mitchell
;   Texas Instruments, Inc
;   January, 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
;
            .equ    Results,0x0200          ; Begining of Results table
;
            .section .text
main:
RESET:      mov     #0xA00,R1               ; Initialize stackpointer
StopWDT:    mov     #WDTPW+WDTHOLD,&WDTCTL  ; Stop watchdog
            bis.b   #BIT0,&P6SEL            ; Enable A/D channel A0
                                            ;
SetupADC12: mov     #SHT0_8+MSC+ADC12ON,&ADC12CTL0 
                                            ; Turn on ADC12, use int. osc.
                                            ; Extend sampling period to avoid
                                            ; overflow
                                            ; Set MSC so conversions triggered
                                            ; automatically
            mov     #SHP+CONSEQ_2,&ADC12CTL1  
                                            ; Use sampling timer, set mode
            mov     #BIT0,&ADC12IE          ; Enable ADC12IFG.0 for ADC12MEM0
            eint                            ; Enable interrupts
            clr     R5                      ; Clear results table pointer
                                            ;
Mainloop:   bis     #ENC,&ADC12CTL0         ; Enable conversions
            bis     #ADC12SC,&ADC12CTL0     ; Start conversions
            bis     #CPUOFF,R2              ; Hold in LPM0 
            nop                             ; Need only for C-SPY
                                            ;
;------------------------------------------------------------------------------ 
ADC12ISR:   ; Interrupt Service Routine for ADC12
;------------------------------------------------------------------------------
            mov     &ADC12MEM0,Results(R5)  ; Move result, IFG is reset
            incd    R5                      ; Increment results table pointer
            and     #0x0E,R5                ; Modulo pointer 
            reti                            ;
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors        
;------------------------------------------------------------------------------
            .section    .vectors
            .org    0x0E,0xFF               ; ADC12 Interrupt Vector
            .word   ADC12ISR                ;
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ;
