#include <io.h>
;******************************************************************************
;   MSP-FET430P440 Demo - Timer_B Toggle P5.1, TBCCR0 upmode ISR, DCO SMCLK 
;
;   Description; This program will toggle P5.1 using software and TB_0 ISR. 
;   Timer_B is configured in an upmode, thus the the timer will overflow when 
;   TBTAR counts to TBCCR0.  In this example, TBCCR0 is loaded with 40000. 
;   Default DCO setting used for TBCLK.
;   ACLK = n/a, MCLK = SMCLK = TBCLK = default DCO
;
;                MSP430F449
;             -----------------
;         /|\|              XIN|-  
;          | |                 |
;          --|RST          XOUT|-
;            |                 |
;            |             P5.1|-->LED
;
;   M.Buccini
;   Texas Instruments, Inc
;   January 2002
;   Adapted for mspgcc by Steve Underwood <steveu@coppice.org>
;******************************************************************************
            .section    .text
RESET:      mov.w   #0xA00,R1               ; Initialize '449 stackpointer
StopWDT:    mov.w   #WDTPW+WDTHOLD,&WDTCTL  ; Stop WDT
SetupTB:    mov.w   #TBSSEL1+TBCLR,&TBCTL   ; SMCLK, clear TAR
SetupC0:    mov.w   #CCIE,&TBCCTL0          ; TBCCR0 interrupt enabled
            mov.w   #40000,&TBCCR0          ; 
SetupP5:    bis.b   #0x02,&P5DIR            ; P5.1 output 
            bis.w   #MC0,&TBCTL             ; Start Timer_a in upmode
            eint                            ; Enable interrupts
                                            ;						  
Mainloop:   bis.w   #CPUOFF,R2              ; CPU off
            nop                             ; Required for C-spy
                                            ;
;------------------------------------------------------------------------------ 
TB0_ISR:    ;Toggle P5.1
;------------------------------------------------------------------------------ 
            xor.b   #0x02,&P5OUT            ; Toggle P5.1
            reti                            ;		 
                                            ;
;------------------------------------------------------------------------------ 
;           Interrupt Vectors Used MSP430x44x         
;------------------------------------------------------------------------------ 
            .section    .vectors
            .org    0x0A,0xFF               ; Timer_B0 Vector
            .word   TB0_ISR                 ; 
            .org    0x1E,0xFF               ; MSP430 RESET Vector
            .word   RESET                   ; 
